@@ -1,13 +1,13 @@
 package POE::Component::IKC::Channel;
 
 ############################################################
-# $Id: Channel.pm,v 1.7 2001/07/13 06:59:45 fil Exp $
+# $Id: Channel.pm,v 1.12 2001/08/02 03:26:50 fil Exp $
 # Based on tests/refserver.perl
 # Contributed by Artur Bergman <artur@vogon-solutions.com>
 # Revised for 0.06 by Rocco Caputo <troc@netrus.net>
 # Turned into a module by Philp Gwyn <fil@pied.nu>
 #
-# Copyright 1999 Philip Gwyn.  All rights reserved.
+# Copyright 1999,2001 Philip Gwyn.  All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 #
@@ -27,7 +27,7 @@ use Data::Dumper;
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_channel);
-$VERSION = '0.13';
+$VERSION = '0.1301';
 
 sub DEBUG { 0 }
 
@@ -123,10 +123,13 @@ sub channel_start
         $heap->{temp_remote_kernel}=$p->{rname};
     } 
     elsif($p->{unix}) {        # we are a client
-        $heap->{remote_kernel}="unix:$p->{unix}:$$:".
+        my $n=$p->{unix};
+        $n=~tr(/\\)(--);
+        $heap->{remote_kernel}="unix:$n:$$:".
                                     fileno($p->{handle});
-        $heap->{temp_remote_kernel}="unix:$p->{unix}" 
-                        if $p->{on_connect};
+        $heap->{temp_remote_kernel}="unix:n" if $p->{on_connect};
+
+
         # we need to have unique aliases for remote kernels
         # so, only the server gets a default name, clients don't
     } 
@@ -520,7 +523,8 @@ sub channel_default
 sub channel_shutdown 
 {
     my $heap = $_[HEAP];
-    DEBUG && warn "$$: Channel will shut down.\n";
+    DEBUG && 
+        warn "$$: *** Channel will shut down.\n";
     _close_wheel($heap);
 }
 
@@ -586,8 +590,10 @@ sub _close_wheel
     my($heap)=@_;
     return unless $heap->{wheel_client};
     if($heap->{pending}) {
+        DEBUG and warn "Defering wheel";
         $heap->{go_away}=1;         # wait until next Flushed
     } else {
+        DEBUG and warn "Deleting wheel";
         delete $heap->{wheel_client};
     }
     
