@@ -1,8 +1,8 @@
-# $Id: Proxy.pm,v 1.5 2001/07/13 06:59:45 fil Exp $
+# $Id: Proxy.pm,v 1.6 2001/07/24 20:45:54 fil Exp $
 package POE::Component::IKC::Proxy;
 
 ##############################################################################
-# $Id: Proxy.pm,v 1.5 2001/07/13 06:59:45 fil Exp $
+# $Id: Proxy.pm,v 1.6 2001/07/24 20:45:54 fil Exp $
 # Copyright 1999 Philip Gwyn.  All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
@@ -21,7 +21,7 @@ use POE::Component::IKC::Specifier;
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_proxy);
-$VERSION = '0.13';
+$VERSION = '0.1301';
 
 sub DEBUG { 0 }
 
@@ -119,8 +119,7 @@ sub _default
     }
 
     DEBUG && print "Proxy $heap->{name}/$state posted\n";
-    foreach my $r_state (@{$heap->{callback}})
-    {
+    foreach my $r_state (@{$heap->{callback}}) {
         $kernel->post('IKC', 'post2', $r_state, $sender, [$state, $args]);
     }
 }
@@ -152,6 +151,10 @@ L<POE>, L<POE::Component::IKC>
 
 
 $Log: Proxy.pm,v $
+Revision 1.6  2001/07/24 20:45:54  fil
+Fixed some win32 things like WSAEAFNOSUPPORT
+Added more tests to t/20_clientlite.t
+
 Revision 1.5  2001/07/13 06:59:45  fil
 Froze to 0.13
 
