@@ -1,13 +1,13 @@
 package POE::Component::IKC::Responder;
 
 ############################################################
-# $Id: Responder.pm,v 1.6 2001/07/13 06:59:45 fil Exp $
+# $Id: Responder.pm,v 1.11 2001/08/02 03:26:50 fil Exp $
 # Based on tests/refserver.perl
 # Contributed by Artur Bergman <artur@vogon-solutions.com>
 # Revised for 0.06 by Rocco Caputo <troc@netrus.net>
 # Turned into a module by Philp Gwyn <fil@pied.nu>
 #
-# Copyright 1999 Philip Gwyn.  All rights reserved.
+# Copyright 1999,2001 Philip Gwyn.  All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 #
@@ -25,7 +25,7 @@ use POE::Component::IKC::Specifier;
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_responder $ikc);
-$VERSION = '0.13';
+$VERSION = '0.1301';
 
 sub DEBUG { 0 }
 
@@ -35,18 +35,18 @@ sub DEBUG { 0 }
 # This is just a convenient way to create only one responder.
 sub create_ikc_responder 
 {
-    return if $ikc;
     __PACKAGE__->spawn();
 }
 
 sub spawn
 {
     my($package)=@_;
+    return if $ikc;
     new POE::Session( $package, [qw(
                       _start _stop
                       request post call raw_message post2
                       remote_error
-                      register unregister default  
+                      register unregister default register_local
                       publish retract subscribe unsubscribe
                       published
                       monitor shutdown 
@@ -61,7 +61,7 @@ sub spawn
 sub _start
 {
     my($kernel, $heap, $session) = @_[KERNEL, HEAP, SESSION];
-    DEBUG and warn "Responder started.\n";
+    DEBUG and warn "$$: Responder started.\n";
     $kernel->alias_set('IKC');              # allow it to be called by name
 
 #    $kernel->signal(INT=>'sig_INT');
@@ -99,6 +99,14 @@ sub register
     $heap->{self}->register($channel, $rid, $aliases);
 }
 
+#----------------------------------------------------
+# Register new aliases for local kernel
+sub register_local
+{
+    my($heap, $aliases) = @_[HEAP, ARG0];
+    $heap->{self}->register_local($aliases);
+}
+
 #----------------------------------------------------
 # Unregister foreign kernels when this disconnect (say)
 sub unregister
@@ -293,6 +301,7 @@ use strict;
 use Carp;
 use POE::Component::IKC::Specifier;
 use POE::Component::IKC::Proxy;
+use POE::Component::IKC::LocalKernel;
 use POE qw(Session);
 
 use Data::Dumper;
@@ -326,12 +335,16 @@ sub new
 sub shutdown
 {
     my($self, $kernel)=@_;
-    DEBUG and warn "Some one wants us to go away... off we go\n";
+    DEBUG and warn "$$: Some one wants us to go away... off we go\n";
+    # kill our alias
     $kernel->alias_remove('IKC');
+    # tell every channel to shutdown
     foreach my $c (values %{$self->{channel}}) {
         $kernel->post($c, 'shutdown');
     }
+    # tell monitors to shutdown
     $self->inform_monitors('*', 'shutdown');
+    # kill pending subscription states
     foreach my $uevent (keys %{$self->{pending_subscription}}) {
         $self->_remove_state($uevent);
     }
@@ -343,7 +356,7 @@ sub request
 {
     my($self, $request)=@_;;
     my($kernel)=@{$self}{qw(poe_kernel)};
-    # DEBUG2 and warn Dumper $request;
+    DEBUG2 and warn Dumper $request;
 
     # We ignore the kernel for now, but we should really use it to decide
     # weither we should run the request or not
@@ -352,19 +365,22 @@ sub request
     eval {
         die "$request->{event} isn't a valid specifier" unless $to;
         my $args=$request->{params};
-        # allow proxied states to have multiple ARGs
-        if($to->{state} eq 'IKC:proxy')   
-        {
+
+        ### allow proxied states to have multiple ARGs
+        if($to->{state} eq 'IKC:proxy') {
             $to->{state}=$args->[0];
             $args=$args->[1];
             DEBUG and warn "IKC proxied request for ", specifier_name($to), "\n";
-        } else
-        {
+        } 
+        else {
             DEBUG and warn "IKC request for ", specifier_name($to), "\n";
             $args=[$args];
         }
+
         # this is where we'd catch a disconnect message
-          
+        # 2001/07 : eh?
+
+
         # find out if the state we want to get at has been published
         if(exists $self->{rsvp}{$to->{session}} and
            exists $self->{rsvp}{$to->{session}}{$to->{state}} and
@@ -374,18 +390,27 @@ sub request
             DEBUG and warn "Allow $to->{session}/$to->{state} is now $self->{rsvp}{$to->{session}}{$to->{state}}\n";
         }
         elsif(not exists $self->{'local'}{$to->{session}}) {
-            die "Session $to->{session} is not available for remote kernels\n", 
-                Dumper $self->published;
-        } 
+            my $p=$self->published;;
+            die "Session '$to->{session}' is not available for remote kernels:",
+                join "\n", '',
+                    map({ "    $_=>[" . join(', ', @{$p->{$_}}) . "]"} keys %$p),
+                    '';
+        }
         elsif(not exists $self->{'local'}{$to->{session}}{$to->{state}}) {
             die "Session '$to->{session}' has not published state '",
                 $to->{state}, "'\n";
         }
 
+        # maybe caller specified #arg?  This got into $msg->{rsvp}, which
+        # went to the remote side, then came back here as $to
+        if(exists $to->{args}) {
+            push @$args, $to->{args};   # it goes on the end
+        }
+
         my $session=$kernel->alias_resolve($to->{session});
         die "Unknown session '$to->{session}'\n" unless $session;
         # warn "No FROM" unless $request->{from};
-        _thunked_post($request->{rsvp}, [$session, $to->{state}, @$args], 
+        _thunked_post($request->{rsvp}, [$session, $to->{state}, @$args],
                           $request->{from}, $request->{wantarray});
     };
 
@@ -429,20 +454,24 @@ sub register
     else {
         DEBUG and warn "$$: Registered remote kernel '$rid'\n";
         $self->{channel}{$rid}=$channel;
-        $self->{remote}{$rid}=[];
-        $self->{alias}{$rid}=$aliases;
+        $self->{remote}{$rid}=[];       # list of proxy sessions
+        $self->{alias}{$rid}=$aliases;  
         $self->{default}||=$rid;
     }
 
 
     foreach my $name (@$aliases) {
+        unless(defined $name) {
+            warn "$$: attempt to register undefined remote kernel alias\n";
+            next;
+        }
         if($self->{kernel}{$name}) {
             DEBUG and warn "$$: Remote alias '$name' already exists\n";
             next;
         }
         DEBUG and warn "$$: Registered alias '$name'\n";
-        $self->{kernel}{$name}=$rid;
-        $self->{remote}{$name}||=[];
+        $self->{kernel}{$name}=$rid;    # find real remote ID
+        $self->{remote}{$name}||=[];    # list of proxy sessions
         $self->{monitors}{$name}||={};
     }
     $self->inform_monitors($rid, 'register');
@@ -450,6 +479,46 @@ sub register
     return 1;
 }
 
+#----------------------------------------------------
+# Register a new alias for the local kernel
+sub register_local
+{
+    my($self, $aliases)=@_;
+    $aliases=[$aliases] if not ref $aliases;
+
+    my($kernel)=@{$self}{qw(poe_kernel)};
+
+    my $rid=$kernel->ID;
+    DEBUG and warn "$$: Registering local kernel '$rid'\n";
+
+    $self->{local_channel}||=POE::Component::IKC::LocalKernel->spawn->ID;
+    my $channel=$self->{local_channel};
+
+    $self->{channel}{$rid}||=$channel;
+    $self->{remote}{$rid}||=[];       # list of proxy sessions
+    $self->{alias}{$rid}||=[];
+
+    # use Data::Dumper;
+    # die Dumper $aliases;
+    foreach my $name (@$aliases) {
+        unless(defined $name) {
+            DEBUG and warn "$$: attempt to register undefined local kernel alias\n";
+            next;
+        }
+        if($self->{kernel}{$name}) {
+            DEBUG and warn "$$: Local kernel alias '$name' already exists\n";
+            next;
+        }
+        DEBUG and warn "$$: Registered local alias '$name'\n";
+        $self->{kernel}{$name}=$rid;    # find real remote ID
+        $self->{remote}{$name}||=[];    # list of proxy sessions
+        $self->{monitors}{$name}||={};
+        push @{$self->{alias}{$rid}}, $name;
+    }
+    return 1;
+}
+
+#----------------------------------------------------
 sub default
 {
     my($self, $name) = @_;
@@ -475,6 +544,7 @@ sub unregister
 {
     my($self, $channel, $rid, $aliases)=@_;
     my($kernel)=@{$self}{qw(poe_kernel)};
+    return unless $rid;
 
     unless($aliases) {
         unless($self->{channel}{$rid}) {    # unregister one alias only
@@ -543,7 +613,7 @@ sub send_msg
 
     my $to=specifier_parse($msg->{event});
     unless($to) {
-        die "Bad state ", Dumper $msg, caller;
+        die "Bad state ", Dumper $msg;
     }
     unless($to) {
         warn "Bad or missing 'event' parameter '$msg->{event}' to poe://IKC/$e\n";
@@ -567,11 +637,12 @@ sub send_msg
     DEBUG and warn "poe://IKC/$e to '", specifier_name($to), "'\n";
 
     # This way the thunk session will proxy a request back to us
-    if($sender and not $msg->{from} and 
+    if($sender and not $msg->{from} and
         # <sungo> Leolo: question. doesnt .13 require bleadPOE?
         # <Leolo> sungo : i can put the offending code into a conditional
         # if you want
-        $self->{poe_kernel}->can('alias_list')) {
+        $self->{poe_kernel}->can('alias_list')) 
+    {
         my $sid=$sender->ID if ref $sender;
         foreach my $a ($self->{poe_kernel}->alias_list($sender)) {
             $sid.=" ($a)";
@@ -579,21 +650,22 @@ sub send_msg
                 $msg->{from}={  kernel=>$self->{poe_kernel}->ID,
                                 session=>$a,
                                 state=>'IKC:proxy',
-                         };
+                             };
                 last;
             }
-        } 
-        unless($msg->{from}) {
-            DEBUG2 and 
+        }
+
+        DEBUG2 and do {
+            unless($msg->{from}) {
                 warn "Session $sid didn't publish anything SENDER isn't set";#, Denter $self->{'local'}, $sender;
-        } else {
-            DEBUG2 and 
+            } else {
                 warn "Session $sid will be thunked";
+            }
         }
     }
 
-    # This is where we should recurse $msg->{params} to turn anything 
-    # extravagant like a subref, $poe_kernel, session etc into a call back to 
+    # This is where we should recurse $msg->{params} to turn anything
+    # extravagant like a subref, $poe_kernel, session etc into a call back to
     # us.
     # $msg->{params}=$self->marshall($msg->{params});
 
@@ -602,8 +674,8 @@ sub send_msg
     unless(@channels)
     {
         warn Dumper $to;
-        warn (($name eq '*') 
-                  ? "$$: Not connected to any foreign kernels.\n" 
+        warn (($name eq '*')
+                  ? "$$: Not connected to any foreign kernels.\n"
                   : "$$: Unknown kernel '$name'.\n");
         return 0;
     }
@@ -627,7 +699,7 @@ sub send_msg
         if($kernel->call($channel, 'send', $msg)) {
             $count++;
             DEBUG2 and warn " done.\n";
-        } 
+        }
         else {
             DEBUG2 and warn " failed.\n";
             $self->{rsvp}{$rsvp->{session}}{$rsvp->{state}}-- if $rsvp;
@@ -635,7 +707,7 @@ sub send_msg
     }
 
     DEBUG2 and warn specifier_name($to), " sent to $count kernel(s).\n";
-    DEBUG and do {warn "$$: send_msg failed!" unless $count};
+    DEBUG and do {warn "$$: send_msg failed!\n" unless $count};
     return $count;
 }
 
@@ -702,7 +774,7 @@ sub demarshall
             my $func=$2;
             my $rk=$1;
             die "need to call $func in $rk";
-            $data=sub {$poe_kernel->post(IKC=>'post', 
+            $data=sub {$poe_kernel->post(IKC=>'post',
                             "poe://$rk/IKC/coderef"=>$func)};
         }
         return $data;
@@ -769,21 +841,19 @@ sub call
     $to="poe://$to"     if $to   and not ref $to   and $to!~/^poe:/;
     $rsvp="poe://$rsvp" if $rsvp and not ref $rsvp and $rsvp!~/^poe:/;
 
+    unless($rsvp) {
+        warn "$$: Missing 'rsvp' parameter in poe://IKC/call\n";
+        return;
+    }
     my $t=specifier_parse($rsvp);
-    unless($t)
-    {
-        if($rsvp)
-        {
-            warn "$$: Bad 'rsvp' parameter '$rsvp' in poe://IKC/call\n";
-        } else
-        {
-            warn "$$: Missing 'rsvp' parameter in poe://IKC/call\n";
-        }
+    unless($t) {
+        warn "$$: Bad 'rsvp' parameter '$rsvp' in poe://IKC/call\n";
         return;
     }
     $rsvp=$t;
     unless($rsvp->{state})
     {
+        DEBUG and warn Dumper $rsvp;
         warn "$$: rsvp state not set in poe://IKC/call\n";
         return;
     }
@@ -797,7 +867,7 @@ sub call
     }
     DEBUG2 and warn "RSVP is ", specifier_name($rsvp), "\n";
 
-    $self->send_msg({params=>$params, 'event'=>$to, 
+    $self->send_msg({params=>$params, 'event'=>$to,
                      rsvp=>$rsvp
                     }, $sender
                    );
@@ -811,10 +881,21 @@ sub publish
 {
     my($self, $session, $states)=@_;
 
-    if(not ref $session) {
-        $session||=$self->{poe_kernel}->ID_lookup($session);
+    if(ref $session) {                  # make sure it's not an object
+        my $kernel=$self->{poe_kernel};
+        if($kernel->can('alias_list')) {
+            # post 0.15 we register as all aliases for session
+            foreach my $alias ($kernel->alias_list($session->ID)) {
+                $self->publish($alias, $states);
+            }
+            return;
+        } 
+        else {
+            # pre 0.15 means that we register as session ID... which is less
+            # then useful
+            $session=$session->ID;
+        }
     }
-
     unless($session) {
         carp "You must specify the session that publishes these states";
         return 0;
@@ -825,7 +906,7 @@ sub publish
 
     die "\$states isn't an array ref" unless ref($states) eq 'ARRAY';
     foreach my $q (@$states) {
-        DEBUG and warn "Published poe://$session/$q\n";
+        DEBUG and warn "Published poe:$session/$q\n";
         $p->{$q}=1;
     }
     return 1;
@@ -857,8 +938,22 @@ sub retract
     my($self, $session, $states)=@_;
 
     my $sid=$session;
-    if(not ref $session) {
-        $sid||=$self->{poe_kernel}->ID_lookup($session);
+
+    if(ref $session) {                  # make sure it's not an object
+
+        my $kernel=$self->{poe_kernel};
+        if($kernel->can('alias_list')) {
+            # post 0.15 we register as all aliases for session
+            foreach my $alias ($kernel->alias_list($session->ID)) {
+                $self->retract($alias, $states);
+            }
+            return;
+        } 
+        else {
+            # pre 0.15 means that we register as session ID... which is less
+            # then useful
+            $sid=$session->ID;
+        }
     }
 
     unless($sid) {
@@ -1189,8 +1284,8 @@ sub DEBUG { 0 }
     $name=~s/\W//g;
     sub thunk
     {
-        my($rsvp, $args, $from, $wantarray)=@_;
-        POE::Session->new(__PACKAGE__, [qw(_start _stop _default)], 
+        # my($rsvp, $args, $from, $wantarray)=@_;
+        POE::Session->new(__PACKAGE__, [qw(_start _stop _default)],
                                        [$name++, @_]
                              );
     }
@@ -1207,7 +1302,7 @@ sub _start
     DEBUG and warn "$name created\n";
 
     if($rsvp) {                         # foreign session wants returned value
-        
+
         DEBUG and do { warn "Wants an array" if $wantarray};
 
         my(@ret, $yes);
@@ -1223,14 +1318,17 @@ sub _start
                 local $"=', ';
                 warn "Posted response '@ret' to ", Dumper $rsvp;
             };
-
-            # This is the POSTBACK            
+            # This is the POSTBACK
             $POE::Component::IKC::Responder::ikc->send_msg(
-                    {params=>($wantarray ? \@ret : $ret[0]), event=>$rsvp}, 
+                    {params=>($wantarray ? \@ret : $ret[0]), event=>$rsvp},
                     $args->[0]);
         }
-    } 
+    }
     else {
+        DEBUG and do {
+            local $"=', ';
+            warn "Posting @$args";
+        };
         $kernel->post(@$args);
     }
 }
@@ -1420,6 +1518,15 @@ kernel, it goes without saying that all it's aliases get unregistered also.
 
 
 
+=head2 C<register_local>
+
+Registers new aliases for local kernel with the responder.  This is done
+internally by C<IKC::Server> and C<IKC::Client>. Will NOT define the default
+kernel.
+
+First and only parameter is an array ref of kernel aliases to be registered.
+
+
 
 
 =head2 C<publish>
@@ -1765,6 +1872,8 @@ This function creates the Responder session and object.  However, you don't
 need to call this directly, because C<IKC::Client> or C<IKC::Server> does
 this for you.
 
+Deprecated, use C<spawn>.
+
 =head1 L<Note about SENDER>
 
 An attempt is made to provide a sane SENDER param to called or posted
@@ -1801,3 +1910,8 @@ Philip Gwyn, <perl-ikc at pied.nu>
 L<POE>, L<POE::Component::IKC::Server>, L<POE::Component::IKC::Client>
 
 =cut
+
+
+$Log: Responder.pm,v $
+Revision 1.11  2001/08/02 03:26:50  fil
+Added documentation.
