@@ -1,18 +1,18 @@
 package POE::Component::IKC::Server;
 
 ############################################################
-# $Id: Server.pm,v 1.5 2001/07/13 06:59:45 fil Exp $
+# $Id: Server.pm,v 1.11 2001/08/02 03:26:50 fil Exp $
 # Based on refserver.perl and preforkedserver.perl
 # Contributed by Artur Bergman <artur@vogon-solutions.com>
 # Revised for 0.06 by Rocco Caputo <troc@netrus.net>
 # Turned into a module by Philp Gwyn <fil@pied.nu>
 #
-# Copyright 1999 Philip Gwyn.  All rights reserved.
+# Copyright 1999,2001 Philip Gwyn.  All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 #
 # Contributed portions of IKC may be copyright by their respective
-# contributors.  
+# contributors.
 
 use strict;
 use Socket;
@@ -27,10 +27,18 @@ require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_server);
-$VERSION = '0.13';
+$VERSION = '0.1301';
 
-sub DEBUG { 0 }
+sub DEBUG { 1 }
 sub DEBUG_USR2 { 1 }
+BEGIN {
+    # http://support.microsoft.com/support/kb/articles/Q150/5/37.asp
+    eval '*WSAEAFNOSUPPORT = sub { 10047};';
+    if($^O eq 'MSWin32') {
+        eval '*EADDRINUSE      = sub { 10048 };';
+    }
+}
+
 
 ###############################################################################
 #----------------------------------------------------
@@ -48,10 +56,10 @@ sub create_ikc_server
     }
 
     create_ikc_responder();
-    POE::Session->new( 
+    POE::Session->new(
                     $params{package}, [qw(
-                        _start _stop error 
-                        accept fork retry waste_time 
+                        _start _stop error
+                        accept fork retry waste_time
                         babysit rogues shutdown
                         sig_CHLD sig_INT sig_USR2
                         )],
@@ -72,6 +80,7 @@ sub _select_define
 {
     my($heap, $on)=@_;
 
+    DEBUG and warn "_select_define";
     my $state;
     my $err="possible redefinition of POE internals";
     $err="failure to bind to port" if $POE::VERSION <= 0.1005;
@@ -82,7 +91,7 @@ sub _select_define
         } else {
             $state=$heap->{wheel}->[5];
         }
-        
+
         unless(defined $state) {
             die "No 'state_accept' in $heap->{wheel}, $err.\n";
         }
@@ -95,7 +104,7 @@ sub _select_define
             $poe_kernel->select_read($heap->{wheel}->{$hndl}, $state);
             $c++;
         }
-    } 
+    }
     else {
         $poe_kernel->select_read($heap->{wheel}->[0], $state);
         $c++;
@@ -112,12 +121,12 @@ sub _start
 {
     my($heap, $params, $kernel) = @_[HEAP, ARG0, KERNEL];
 
-    # monitor for shutdown events.  
-    # this is the best way to get IKC::Responder to tell us about the 
+    # monitor for shutdown events.
+    # this is the best way to get IKC::Responder to tell us about the
     # shutdown
     $kernel->post(IKC=>'monitor', '*', {shutdown=>'shutdown'});
 
-    my $n='unknown';     
+    my $n='unknown';
     my %wheel_p=(
         Reuse          => 'yes',        # and allow immediate reuse of the port
         SuccessState   => 'accept',     # generating this event on connection
@@ -129,7 +138,7 @@ sub _start
         $wheel_p{BindAddress}=$params->{unix};
         $heap->{unix}=$params->{unix};
         unlink $heap->{unix};           # blindly do this ?
-    } 
+    }
     else {
         $n="$params->{ip}:$params->{port}";
         $wheel_p{BindPort}= $params->{port};
@@ -142,6 +151,18 @@ sub _start
     $heap->{name}=$params->{name};
     $heap->{aliases}=$params->{aliases};
 
+    # set up local names for kernel
+    my @names=($heap->{name});
+    if($heap->{aliases}) {
+        if(ref $heap->{aliases}) {
+            push @names, @{$heap->{aliases}};
+        } else {
+            push @names, $heap->{aliases};
+        }
+    }
+
+    $kernel->post(IKC=>'register_local', \@names);
+
     return unless $params->{processes};
 
     # Delete the SocketFactory's read select in the parent
@@ -158,7 +179,7 @@ sub _start
     $heap->{'failed forks'} = 0;
     $heap->{verbose}=$params->{verbose}||0;
     $heap->{"max connections"}=$params->{connections}||1;
-                                        
+
     $heap->{'is a child'} = 0;          # change behavior for children
     foreach (2..$params->{processes}) { # fork the initial set of children
         $kernel->yield('fork');
@@ -243,7 +264,7 @@ sub babysit
                 if($time and $time > 600000) { # arbitrary limit of 10 minutes
                     $rogues{$pid}=$table{$pid};
                     # DEBUG and 
-                    warn "$$: $pid hass gone rogue, time=$time ms\n";
+                        warn "$$: $pid hass gone rogue, time=$time ms\n";
                 } else {
                     warn "$$: $pid time=$time ms\n";
                     $ok{$pid}=1;
@@ -358,7 +379,7 @@ eval {
 sub _stop 
 {
     my($kernel, $heap) = @_[KERNEL, HEAP];
-    check_kernel($kernel, $heap->{'is a child'});
+
                                         # kill the child servers
     if($heap->{children}) {
         foreach (keys %{$heap->{children}}) {
@@ -369,7 +390,9 @@ sub _stop
     if($heap->{unix}) {
         unlink $heap->{unix};
     }
-    DEBUG && print "$$: server is stoping\n";
+    DEBUG && 
+        warn "$$: server is stoping\n";
+    # DEBUG_USR2 and check_kernel($kernel, $heap->{'is a child'}, 1);
 }
 
 #------------------------------------------------------------------------------
@@ -390,14 +413,23 @@ sub shutdown
 # error occurs while initializing the factory's listening socket, it
 # will exit anyway.
 
-sub error 
+sub error
 {
     my ($heap, $operation, $errnum, $errstr) = @_[HEAP, ARG0, ARG1, ARG2];
-    # TODO : post to monitors
-    warn __PACKAGE__, " encountered $operation error $errnum: $errstr\n";
+    my $ignore;
     if($errnum==EADDRINUSE) {       # EADDRINUSE
+        warn "$$: Address in use\n";
         $heap->{'die'}=1;
         delete $heap->{wheel};
+        $ignore=1;
+    } elsif($errnum==WSAEAFNOSUPPORT) {
+        # Address family not supported by protocol family.
+        # we get this error, yet nothing bad happens... oh well
+        $ignore=1;
+    }
+    unless($ignore) {
+        # TODO : post to monitors
+        warn __PACKAGE__, " $$: encountered $operation error $errnum: $errstr\n";
     }
 }
 
@@ -437,13 +469,15 @@ sub accept
     if ($heap->{'is a child'}) {
 
         if (--$heap->{connections} < 1) {
-            DEBUG and warn "$$: Game over\n";
+            DEBUG and 
+                warn "$$: ************* Game over\n";
             $kernel->delay('waste_time');
             delete $heap->{wheel};
             check_kernel($kernel);
 #            $kernel->yield('_stop');
         } else {
-            DEBUG and warn "$$: $heap->{connections} left\n";
+            DEBUG and 
+                warn "$$: $heap->{connections} left\n";
         }
     } else {
         warn "$$: Master client got a connect!  this sucks!\n";
@@ -600,18 +634,24 @@ sub check_kernel
 {
     my($kernel, $child, $signal)=@_;
     if(ref $kernel) {
+        # 2 = KR_HANDLES
         # 3 = KR_STATES
         # 8 = KR_ALARMS
-        # 2 = KR_HANDLES
         # 14 = KR_EXTRA_REFS
-    
-        warn( "$$ ----- Kernel Activity -----\n",
-               "| States : ", scalar(@{$kernel->[3]}), "\n",
-               "| Alarms : ", scalar(@{$kernel->[8]}), "\n",
-               "| Files  : ", scalar(keys(%{$kernel->[2]})), "\n",
-               "| Extra  : ", $kernel->[14], "\n",
-               "$$ ---------------------------\n"
-             ) if $signal; 
+
+        # 0 = HND_HANDLE
+        warn( "$$: ,----- Kernel Activity -----\n",  
+              "$$: | States : ", scalar(@{$kernel->[3]}), "\n",
+              "$$: | Alarms : ", scalar(@{$kernel->[8]}), "\n",
+              "$$: | Files  : ", scalar(keys(%{$kernel->[2]})), "\n",
+              "$$: |   `--> : ", join( ', ',
+                               sort { $a <=> $b }
+                               map { fileno($_->[0]) }
+                               values(%{$kernel->[2]})
+                             ),   "\n",
+              "$$: | Extra  : ${$kernel->[14]}\n",
+              "$$: `---------------------------\n",
+         );
         if($child) {
             foreach my $q (@{$kernel->[8]}) {
                 warn "************ Alarm for ", join '/', @{$q->[0][2]{$q->[2]}};
@@ -685,7 +725,9 @@ socket is attempted!  Buyer beware.
 
 Local kernel name.  This is how we shall "advertise" ourself to foreign
 kernels. It acts as a "kernel alias".  This parameter is temporary, pending
-the addition of true kernel names in the POE core.
+the addition of true kernel names in the POE core.  This name, and all
+aliases will be registered with the responder so that you can post to them
+as if they were remote.
 
 =item C<aliases>
 
